import haxe.Timer;
import js.Browser.console;
import tink.state.*;

using tink.CoreApi;

class Test {
	static function debug(msg) console.log(msg);
	static function info(msg) console.info('%c$msg', "color: blue");
	static function error(msg) console.error('%c$msg', "color: red");

	static function main() {
		final _signal:SignalTrigger<Int> = Signal.trigger();
		final signal = _signal.asSignal();
		var signalHandled = false;
		signal.handle((v:Int) -> {
			debug('signal handler 1 v=$v');
			if (v == 2) throw "signal: oh no!";
		});
		signal.handle(v -> {
			debug('signal handler 2 v=$v');
			signalHandled = true;
		});

		final _state = new State(-1);
		final state = _state.observe();
		var stateHandled = false;
		state.bind((v:Int) -> {
			if (v < 0) return;
			debug('state binding 1 v=$v');
			if (v == 2) throw "state: oh no!";
		});
		state.bind(v -> {
			if (v < 0) return;
			debug('state binding 2 v=$v');
			stateHandled = true;
		});

		var counter = 0;
		for (i in 0...3) Timer.delay(() -> {
			info('timer $i');
			++counter;
			signalHandled = stateHandled = false;
			Timer.delay(() -> {
				try	_signal.trigger(counter) catch (err) error('Signal trigger exception caught: $err');
				if (!signalHandled) error("Signal handler 2 was not called");
			}, 1);
			Timer.delay(() -> _state.set(counter), 1);
			Timer.delay(() -> if (!stateHandled) error("State binding 2 was not called"), 50);
		}, i * 1000 + 10);
	}
}
